import {
  Entity,
  Column,
  PrimaryGeneratedColumn,
  BeforeInsert,
  OneToMany,
} from 'typeorm';
import * as bcrypt from 'bcryptjs';
import * as jwt from 'jsonwebtoken';

import { Game_PersonEntity } from '../game/game.entity';
import { Exclude } from 'class-transformer';

/*
UserEntity reflects database table.

Before handling password to database we encrypt it with bcrypt.
password field will be excluded unless we call repository relation.

token will be created when user registers or logs in to the system.
*/

@Entity('Person')
export class PersonEntity {
  @PrimaryGeneratedColumn('uuid') id: string;
  @Column({ type: 'text', unique: true }) name: string;

  @Exclude()
  @Column('text')
  password: string;

  @OneToMany(type => Game_PersonEntity, game_persons => game_persons.person)
  game_persons: Game_PersonEntity[];

  // hashes the password before inserting it to database
  @BeforeInsert()
  async hashPassword() {
    this.password = await bcrypt.hash(this.password, 10);
  }

  // returns username and associated token
  tokenObject() {
    const { name, token } = this;
    return { name, token };
  }

  async comparePassword(attempt: string) {
    return await bcrypt.compareSync(attempt, this.password);
  }

  // creates token from id and name, it will be created through jsonwebtoken and .env SECRET field
  private get token() {
    const { id, name } = this;
    return jwt.sign(
      {
        id,
        name,
      },
      process.env.SECRET,
      { expiresIn: '7d' },
    );
  }
}
